@@ -1,4 +1,4 @@
-# Copyright 2009-2013 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013, 2014 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -14,7 +14,8 @@
 #    You should have received a copy of the GNU Affero General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import os.path
+from __future__ import print_function, unicode_literals, absolute_import, division
+from six.moves import input
 
 from reahl.dev.devdomain import Project
 from reahl.dev.devshell import WorkspaceCommand
@@ -33,16 +34,16 @@ class ServeCurrentProject(WorkspaceCommand):
         if args:
             config_directory = args[0]
 
-        print '\nUsing config from %s\n' % config_directory
+        print('\nUsing config from %s\n' % config_directory)
             
         with project.paths_set():
             reahl_server = self.start_server(options, config_directory)
 
-            print '\n\nServing http on port %s, https on port %s (config=%s)' % \
-                                     (options.port, int(options.port)+363, config_directory)
-            print '\nPress <Enter> to terminate\n\n'
+            print('\n\nServing http on port %s, https on port %s (config=%s)' % \
+                                     (options.port, int(options.port)+363, config_directory))
+            print('\nPress <Enter> to terminate\n\n')
     
-            raw_input()        
+            input()        
             reahl_server.stop()
         return 0
 
