@@ -1,4 +1,4 @@
-# Copyright 2009-2013 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013, 2014 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -14,35 +14,32 @@
 #    You should have received a copy of the GNU Affero General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from threading import Thread, Event, Timer
-import threading
+from __future__ import print_function, unicode_literals, absolute_import, division
+import six
+from threading import Event
+from threading import Thread
 import select
 from wsgiref import simple_server
 import sys
 import traceback
 import socket
 import ssl
-import tempfile
-import os.path 
 from contextlib import contextmanager
-import mimetypes
-from datetime import datetime
 import logging
 import functools
 import pkg_resources
 
-from webob import Request, Response
-from webob.exc import HTTPNotFound, HTTPInternalServerError
+from webob import Request
+from webob.exc import HTTPInternalServerError
 
+from reahl.component.exceptions import ProgrammerError
 from reahl.component.context import ExecutionContext
 from reahl.component.config import StoredConfiguration
-from reahl.web.fw import ReahlWebApplication
+from reahl.component.py3compat import ascii_as_bytes_or_str
+from reahl.web.fw import ReahlWSGIApplication
 
 class WrappedApp(object):
-    """A class in which to wrap a WSGI App and ensure it yields str elements.
-
-    This is necessary, since Templates sometimes return unicode and
-    wsgiref implementation we use in development expects str.
+    """A class in which to wrap a WSGI app, allowing catching of exceptions in the wrapped app.
     """
     def __init__(self, wrapped):
         self.wrapped = wrapped
@@ -52,28 +49,31 @@ class WrappedApp(object):
     def __call__(self, environ, start_response):
         app = self.wrapped
         
-        request = Request(environ)
+        request = Request(environ, charset='utf-8')
         
         self.exception = None
         self.traceback = None
         try:
-            to_return = ''
+            to_return = b''
             for i in app(environ, start_response):
-                to_return += str(i)
+                to_return += i 
         except socket.error:
-            to_return = ''
-            for i in HTTPInternalServerError()(environ, start_response):
+            to_return = b''
+            for i in HTTPInternalServerError(charset=ascii_as_bytes_or_str('utf-8'))(environ, start_response):
                 to_return += i
         except:
-            to_return = ''
+            to_return = b''
             (_, self.exception, self.traceback) = sys.exc_info()
-            traceback_html = traceback.format_exc(self.traceback)
-            for i in HTTPInternalServerError(content_type='text/plain', charset=None, body=traceback_html)(environ, start_response):
+            try:
+                traceback_html = six.text_type(traceback.format_exc(self.traceback))
+            except Exception as ex:
+                traceback_html = 'Exception happened while trying to render traceback: %s' % ex
+            for i in HTTPInternalServerError(content_type=ascii_as_bytes_or_str('text/plain'), charset=ascii_as_bytes_or_str('utf-8'), unicode_body=traceback_html)(environ, start_response):
                 to_return += i
         yield to_return
 
-    def set_new_wrapped(self, new_webapp):
-        self.wrapped = new_webapp
+    def set_new_wrapped(self, new_wsgi_app):
+        self.wrapped = new_wsgi_app
 
     def __getattr__(self, name):
         return getattr(self.wrapped, name)
@@ -83,11 +83,12 @@ class WrappedApp(object):
 
     def report_exception(self):
         if self.exception:
-            raise self.exception, None, self.traceback
+            six.reraise(self.exception.__class__, self.exception, self.traceback)
             
     def clear_exception(self):
         self.exception = None
 
+
 class NoopApp(object):
     def __init__(self, config=None):
         pass
@@ -103,6 +104,7 @@ class NoopApp(object):
     def clear_exception(self):
         pass
 
+
 class LoggingRequestHandler(simple_server.WSGIRequestHandler):
     def log_message(self, format, *args):
         message = format % args
@@ -115,20 +117,20 @@ class LoggingRequestHandler(simple_server.WSGIRequestHandler):
         #   but there really is nothing to read on the socket, and our server will block here.
         # All this timeout stuff justs safeguards against that eventuality - which is a hard thing
         #   to debug if it happens.
-        self.rfile._sock.settimeout(3)  
+        if six.PY2:
+            self.rfile._sock.settimeout(3)  
+        else:
+            self.connection.settimeout(3)
 
     def handle(self):
         try:
             simple_server.WSGIRequestHandler.handle(self)
         except socket.timeout:
-            message = u'Server socket timed out waiting to receive the request. This may happen if the server mistakenly deduced that there were requests waiting for it when there were not. Such as when chrome prefetches things, etc.'
+            message = 'Server socket timed out waiting to receive the request. This may happen if the server mistakenly deduced that there were requests waiting for it when there were not. Such as when chrome prefetches things, etc.'
             logging.getLogger(__name__).warn(message)
 
     def finish_response(self):
-        try:
-            simple_server.WSGIRequestHandler.finish_response()
-        except socket.error, ex:
-            import pdb; pdb.set_trace()
+        simple_server.WSGIRequestHandler.finish_response()
 
 
 class SSLWSGIRequestHandler(LoggingRequestHandler):
@@ -140,8 +142,8 @@ class SSLWSGIRequestHandler(LoggingRequestHandler):
 
 class ReahlWSGIServer(simple_server.WSGIServer):
     @classmethod
-    def make_server(cls, host, port, reahl_webapp):
-        httpd = simple_server.make_server(host, port, reahl_webapp, server_class=cls, 
+    def make_server(cls, host, port, reahl_wsgi_app):
+        httpd = simple_server.make_server(host, port, reahl_wsgi_app, server_class=cls, 
                                           handler_class=LoggingRequestHandler)
         return httpd
 
@@ -149,23 +151,16 @@ class ReahlWSGIServer(simple_server.WSGIServer):
         simple_server.WSGIServer.__init__(self, server_address, RequestHandlerClass)
         self.allow_reuse_address = True
         
-    def serve_async(self, threaded=False):
+    def serve_async(self):
         if self.requests_waiting(0.01):
-            context = ExecutionContext.get_context()
-            if threaded:
-                thread = Thread(target=functools.partial(self.handle_waiting_request, context))
-                thread.daemon = True
-                thread.start()
-            else:
-                self.handle_waiting_request(context)
+            self.handle_waiting_request()
 
-    def handle_waiting_request(self, context):
-        with context:
-            self.handle_request()
-            try:
-                self.get_app().report_exception()
-            finally:
-                self.get_app().clear_exception()
+    def handle_waiting_request(self):
+        self.handle_request()
+        try:
+            self.get_app().report_exception()
+        finally:
+            self.get_app().clear_exception()
     
     def requests_waiting(self, timeout):
         i, o, w = select.select([self.socket],[],[],timeout)
@@ -174,9 +169,9 @@ class ReahlWSGIServer(simple_server.WSGIServer):
 
 class SSLCapableWSGIServer(ReahlWSGIServer):
     @classmethod
-    def make_server(cls, host, port, certfile, reahl_webapp):
+    def make_server(cls, host, port, certfile, reahl_wsgi_app):
         cls.certfile = certfile
-        httpd = simple_server.make_server(host, port, reahl_webapp, server_class=cls, 
+        httpd = simple_server.make_server(host, port, reahl_wsgi_app, server_class=cls, 
                                           handler_class=SSLWSGIRequestHandler)
         return httpd
 
@@ -185,28 +180,32 @@ class SSLCapableWSGIServer(ReahlWSGIServer):
         self.socket = ssl.wrap_socket(self.socket, certfile=self.certfile)
         
     def get_request(self):
-        #This method is shamelessly copied from WerkZeug (and changed)
-        class _SSLConnectionFix(object):
-            #This class is shamelessly copied from WerkZeug
-            """Wrapper around SSL connection to provide a working makefile()."""
+        if six.PY2:
+            #This method is shamelessly copied from WerkZeug (and changed)
+            class _SSLConnectionFix(object):
+                #This class is shamelessly copied from WerkZeug
+                """Wrapper around SSL connection to provide a working makefile()."""
 
-            def __init__(self, con):
-                self._con = con
+                def __init__(self, con):
+                    self._con = con
 
-            def makefile(self, mode, bufsize):
-                return socket._fileobject(self._con, mode, bufsize)
+                def makefile(self, mode, bufsize):
+                    return socket._fileobject(self._con, mode, bufsize)
 
-            def __getattr__(self, attrib):
-                return getattr(self._con, attrib)
+                def __getattr__(self, attrib):
+                    return getattr(self._con, attrib)
 
-        con, info = self.socket.accept()
-        con = _SSLConnectionFix(con)
+            con, info = self.socket.accept()
+            con = _SSLConnectionFix(con)
+        else:
+            con, info = self.socket.accept()
+        
         return con, info
 
     def finish_request(self, request, client_address):
         try:
             ReahlWSGIServer.finish_request(self, request, client_address)
-        except ssl.SSLError, ex:
+        except ssl.SSLError as ex:
             pass
 
 class Handler(object):
@@ -225,6 +224,7 @@ class Handler(object):
     def install(self, reahl_server):
         self.reahl_server = reahl_server
         def wrapped_execute(command, params):
+            exceptions=[]
             results = []
             started = Event()
             def doit():
@@ -232,7 +232,8 @@ class Handler(object):
                     started.set()
                     r = self.original_execute(command, params)
                     results.append(r)
-                except Exception, e:
+                except Exception as e:
+                    exceptions.append(e)
                     raise
                 finally:
                     results.append(None)
@@ -241,7 +242,9 @@ class Handler(object):
             started.wait()
 
             self.reahl_server.serve_until(lambda: not command_thread.is_alive())
-            command_thread.join()
+            if exceptions:
+                raise Exception(exceptions[0])
+            command_thread.join(5)
             return results[0]
         self.command_executor.execute = wrapped_execute
 
@@ -266,9 +269,9 @@ class ReahlWebServer(object):
         config.configure()
         return cls(config, port)
 
-    def set_app(self, new_webapp):
-        """Changes the currently served application to `new_webapp`."""
-        self.reahl_webapp.set_new_wrapped(new_webapp)
+    def set_app(self, new_wsgi_app):
+        """Changes the currently served application to `new_wsgi_app`."""
+        self.reahl_wsgi_app.set_new_wrapped(new_wsgi_app)
 
     def set_noop_app(self):
         self.set_app(NoopApp())
@@ -277,33 +280,39 @@ class ReahlWebServer(object):
         self.in_seperate_thread = None
         self.running = False
         self.handlers = {}
-        certfile = pkg_resources.resource_filename(__name__, u'reahl_development_cert.pem')
-        self.reahl_webapp = WrappedApp(ReahlWebApplication(config))
+        self.httpd_thread = None
+        certfile = pkg_resources.resource_filename(__name__, 'reahl_development_cert.pem')
+        self.reahl_wsgi_app = WrappedApp(ReahlWSGIApplication(config))
         try:
             https_port = port+363
-            self.httpd = ReahlWSGIServer.make_server('', port, self.reahl_webapp)
-            self.httpsd = SSLCapableWSGIServer.make_server('', https_port, certfile, self.reahl_webapp)
-        except socket.error, ex:
-            message = (u'Caught socket.error: %s\nThis means that another process is using one of these ports: %s, %s. ' % (ex, port, https_port)) \
-                     +u'\nIf this happens while running tests, it probably means that a browser client did not close its side of a connection to a previous server you had running - and that the server socket now sits in TIME_WAIT state. Is there perhaps a browser hanging around from a previous run? I have no idea how to fix this automatically... see http://hea-www.harvard.edu/~fine/Tech/addrinuse.html' \
+            self.httpd = ReahlWSGIServer.make_server('', port, self.reahl_wsgi_app)
+            self.httpsd = SSLCapableWSGIServer.make_server('', https_port, certfile, self.reahl_wsgi_app)
+        except socket.error as ex:
+            message = ('Caught socket.error: %s\nThis means that another process is using one of these ports: %s, %s. ' % (ex, port, https_port)) \
+                     +'\nIf this happens while running tests, it probably means that a browser client did not close its side of a connection to a previous server you had running - and that the server socket now sits in TIME_WAIT state. Is there perhaps a browser hanging around from a previous run? I have no idea how to fix this automatically... see http://hea-www.harvard.edu/~fine/Tech/addrinuse.html' \
                       
             raise AssertionError(message)
 
-    def main_loop(self):
-        while self.running:
-            self.httpd.serve_async(threaded=True)
-            self.httpsd.serve_async(threaded=True)
+    def main_loop(self, context):
+        with context:
+            while self.running:
+                self.httpd.serve_async()
+                self.httpsd.serve_async()
 
     def start_thread(self):
         assert not self.running
         self.running = True
-        self.httpd_thread = Thread(target=self.main_loop)
+        self.httpd_thread = Thread(target=functools.partial(self.main_loop, ExecutionContext.get_context()))
         self.httpd_thread.daemon = True
         self.httpd_thread.start()
 
-    def stop_thread(self):
+    def stop_thread(self, join=True):
         self.running = False
-        self.httpd_thread.join()
+        if self.httpd_thread and join:
+            self.httpd_thread.join(5)
+            if self.httpd_thread.is_alive():
+                raise ProgrammerError('Timed out after 5 seconds waiting for httpd serving thread to end')
+        self.httpd_thread = None
 
     def start(self, in_seperate_thread=True, connect=False):
         """Starts the webserver and web application.
@@ -311,7 +320,7 @@ class ReahlWebServer(object):
            :keyword in_seperate_thread: If False, the server handles requests in the same thread as your tests.
            :keyword connect: If True, also connects to the database.
         """
-        self.reahl_webapp.start(connect=connect)
+        self.reahl_wsgi_app.start(connect=connect)
         if in_seperate_thread:
             self.start_thread()
         self.in_seperate_thread = in_seperate_thread
@@ -320,34 +329,20 @@ class ReahlWebServer(object):
         """Stops the webserver and web application from running."""
         if self.running:
             self.stop_thread()
-        self.reahl_webapp.stop()
-        self.shutdown_socket(self.httpd.socket)
-        self.shutdown_socket(self.httpsd.socket)
+        self.reahl_wsgi_app.stop()
+        self.httpd.server_close()
+        self.httpsd.server_close()
     
-    def shutdown_socket(self, socket_to_shutdown):
-        """ On windows, the shutdown of a socket does not work as we expect.
-            http://stackoverflow.com/questions/409783/socket-shutdown-vs-socket-close
-            This method tries the shutdown, and if it cannot, tries to close it.
-        """
-        try:
-            socket_to_shutdown.shutdown(socket.SHUT_RDWR)
-        except socket.error, e:
-            if e.errno == 10057:
-                socket_to_shutdown.close();
-            else:
-                raise
-            
     def requests_waiting(self, timeout):
         return self.httpd.requests_waiting(timeout) or self.httpsd.requests_waiting(timeout/10)
 
     def serve_until(self, done):
-        count = 0
-        while not (done() or self.reahl_webapp.has_uncaught_exception()):
+        while not (done() or self.reahl_wsgi_app.has_uncaught_exception()):
             self.httpd.serve_async()
             self.httpsd.serve_async()
 
     def serve(self, timeout=0.01):
-        """Call this method once, to have the server handle all waiting requests in hte calling thread."""
+        """Call this method once to have the server handle all waiting requests in the calling thread."""
         def done():
             return not self.requests_waiting(timeout)
         self.serve_until(done)
@@ -369,18 +364,22 @@ class ReahlWebServer(object):
             handler.reinstall()
 
     @contextmanager
-    def in_background(self):
+    def in_background(self, wait_till_done_serving=True):
         """Returns a context manager. Within the context of this context manager, the webserver is temporarily run
            in a separate thread. After the context managed by this context manager is exited, the server reverts to 
            handling requests in the current (test) thread.
+
+           :keyword wait_till_done_serving: If True, wait for the server to finish its background job before exiting the context block.
         """
         self.restore_handlers()
         self.start_thread()
         try:
             yield
         finally:
-            self.stop_thread()
-            self.reinstall_handlers()
+            try:
+                self.stop_thread(join=wait_till_done_serving)
+            finally:
+                self.reinstall_handlers()
 
 
 
